module Types where

import Data.Char (toLower)
import Data.List (find)
import qualified Data.Maybe

data HTML = Text String | Node {htmlType :: Tag, attributes :: [Attr], children :: [HTML]} | Root {ch :: [HTML]} deriving (Eq, Show, Read)

data ParsedTag = OpeningTag {tag :: Tag, attrs :: [Attr]} | ClosingTag {tag :: Tag} | TextTag {text :: String} deriving (Eq, Show, Read)

data Attr = K String | KV String String deriving (Eq, Show, Read)

data Tag = Normal {name :: String} | Void {name :: String} | Invalid {name :: String} deriving (Eq, Show, Read)

getTag :: String -> Tag
getTag t = Data.Maybe.fromMaybe (Invalid $ map toLower t) (find ((== map toLower t) . name) tags)

tags :: [Tag]
tags =
  [ Void "!doctype",
    Normal "a",
    Normal "abbr",
    Normal "address",
    Void "area",
    Normal "article",
    Normal "aside",
    Normal "audio",
    Normal "b",
    Void "base",
    Normal "bdi",
    Normal "bdo",
    Normal "blockquote",
    Normal "body",
    Void "br",
    Normal "button",
    Normal "canvas",
    Normal "caption",
    Normal "cite",
    Normal "code",
    Void "col",
    Normal "colgroup",
    Normal "data",
    Normal "datalist",
    Normal "dd",
    Normal "del",
    Normal "details",
    Normal "dfn",
    Normal "dialog",
    Normal "div",
    Normal "dl",
    Normal "dt",
    Normal "em",
    Void "embed",
    Normal "fieldset",
    Normal "figure",
    Normal "footer",
    Normal "form",
    Normal "h1",
    Normal "h2",
    Normal "h3",
    Normal "h4",
    Normal "h5",
    Normal "h6",
    Normal "head",
    Normal "header",
    Normal "hgroup",
    Void "hr",
    Normal "html",
    Normal "i",
    Normal "iframe",
    Void "img",
    Void "input",
    Normal "ins",
    Normal "kbd",
    Normal "keygen",
    Normal "label",
    Normal "legend",
    Normal "li",
    Void "link",
    Normal "main",
    Normal "map",
    Normal "mark",
    Normal "menu",
    Void "menuitem",
    Void "meta",
    Normal "meter",
    Normal "nav",
    Normal "noscript",
    Normal "object",
    Normal "ol",
    Normal "optgroup",
    Normal "option",
    Normal "output",
    Normal "p",
    Void "param",
    Normal "pre",
    Normal "progress",
    Normal "q",
    Normal "rb",
    Normal "rp",
    Normal "rt",
    Normal "rtc",
    Normal "ruby",
    Normal "s",
    Normal "samp",
    Normal "script",
    Normal "section",
    Normal "select",
    Normal "small",
    Void "source",
    Normal "span",
    Normal "strong",
    Normal "style",
    Normal "sub",
    Normal "summary",
    Normal "sup",
    Normal "table",
    Normal "tbody",
    Normal "td",
    Normal "template",
    Normal "textarea",
    Normal "tfoot",
    Normal "th",
    Normal "thead",
    Normal "time",
    Normal "title",
    Normal "tr",
    Void "track",
    Normal "u",
    Normal "ul",
    Normal "var",
    Normal "video",
    Void "wbr"
  ]